@@ -27,34 +27,39 @@ rm(prefix)
 setAs("dtCMatrix", "dtTMatrix",
       function(from) .Call(Csparse_to_Tsparse, from, TRUE))
 
-setAs("CsparseMatrix", "denseMatrix",
-      function(from) {
-	  ## |-> cholmod_C -> cholmod_dense -> chm_dense_to_dense
-	  cld <- getClassDef(class(from))
-	  if (extends(cld, "generalMatrix"))
-	      .Call(Csparse_to_dense, from)
-	  else {
-	      ## Csparse_to_dense  loses symmetry and triangularity properties.
-	      ## With suitable changes to chm_dense_to_SEXP (../src/chm_common.c)
-	      ## we could do this in C code -- or do differently in C {FIXME!}
-	      if (extends(cld, "triangularMatrix") && from@diag == "U")
-		  from <- .Call(Csparse_diagU2N, from)
-	      as(.Call(Csparse_to_dense, from), # -> "[dln]geMatrix"
-		 paste0(.M.kind(from, cld),
-			.dense.prefixes[.M.shape(from, cld)], "Matrix"))
-	  }
-      })
+if(FALSE) ## old version
+C2dense <- function(from) {
+    ## |-> cholmod_C -> cholmod_dense -> chm_dense_to_dense
+    cld <- getClassDef(class(from))
+    if (extends(cld, "generalMatrix"))
+	.Call(Csparse_to_dense, from, FALSE)
+    else { ## "triangular" or "symmetric" :
+        tri <- extends(cld, "triangularMatrix")
+	## Csparse_to_dense  loses symmetry and triangularity properties.
+	## With suitable changes to chm_dense_to_SEXP (../src/chm_common.c)
+	## we could do this in C code -- or do differently in C {FIXME!}
+	if (tri && from@diag == "U")
+	    from <- .Call(Csparse_diagU2N, from)
+	as(.Call(Csparse_to_dense, from, symm = !tri), # -> "[dln]geMatrix"
+	   paste0(.M.kindC(cld),
+		  .dense.prefixes[if(tri) "t" else "s"], "Matrix"))
+    }
+}
+C2dense <- function(from) .Call(Csparse_to_dense, from, NA_integer_)
+
+setAs("CsparseMatrix", "denseMatrix", C2dense)
 
 ## special cases (when a specific "to" class is specified)
-setAs("dgCMatrix", "dgeMatrix",
-      function(from) .Call(Csparse_to_dense, from))
+setAs("dgCMatrix", "dgeMatrix",   function(from) .Call(Csparse_to_dense, from,  0L))
+setAs("dsCMatrix", "denseMatrix", function(from) .Call(Csparse_to_dense, from,  1L))
+setAs("dtCMatrix", "denseMatrix", function(from) .Call(Csparse_to_dense, from, -1L))
 
 setAs("dgCMatrix", "vector", function(from) .Call(Csparse_to_vector, from))
 setAs("dsCMatrix", "vector", function(from) .Call(Csparse_to_vector, from))
-setMethod("as.vector", signature(x = "dgCMatrix", mode = "missing"),
-	  function(x, mode) .Call(Csparse_to_vector, x))
-setMethod("as.vector", signature(x = "dsCMatrix", mode = "missing"),
-	  function(x, mode) .Call(Csparse_to_vector, x))
+setMethod("as.vector", "dgCMatrix",
+	  function(x, mode) as.vector(.Call(Csparse_to_vector, x), mode))
+setMethod("as.vector", "dsCMatrix",
+	  function(x, mode) as.vector(.Call(Csparse_to_vector, x), mode))
 ## could do these and more for as(., "numeric") ... but we *do* recommend  as(*,"vector"):
 ## setAs("dgCMatrix", "numeric", Csp2vec)
 ## setAs("dsCMatrix", "numeric", Csp2vec)
@@ -63,13 +68,38 @@ setMethod("as.vector", signature(x = "dsCMatrix", mode = "missing"),
 ## cholmod_sparse_to_dense converts symmetric storage to general
 ## storage so symmetric classes are ok for conversion to matrix.
 ## unit triangular needs special handling
-.dxC2mat <- function(from, chkUdiag=TRUE) .Call(Csparse_to_matrix, from, chkUdiag)
-setAs("dgCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, FALSE))
-setAs("dsCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, FALSE))
-setAs("dtCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, TRUE))
+##' exported
+.dxC2mat <- function(from, chkUdiag=TRUE) .Call(Csparse_to_matrix, from, chkUdiag, NA)
+setAs("dgCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, FALSE, FALSE))
+setAs("dsCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, FALSE, TRUE))
+setAs("dtCMatrix", "matrix", function(from) .Call(Csparse_to_matrix, from, TRUE,  FALSE))
 ## NB: Would *not* be ok for l*Matrix or n*Matrix,
 ## --------- as cholmod coerces to "REAL" aka "double"
 
+.m2dgC <- function(from) {
+    if(!is.double(from)) storage.mode(from) <- "double"
+    .Call(matrix_to_Csparse, from, "dgCMatrix")
+}
+.m2lgC <- function(from) {
+    if(!is.logical(from)) storage.mode(from) <- "logical"
+    .Call(matrix_to_Csparse, from, "lgCMatrix")
+}
+.m2ngC <- function(from) {
+    if(!is.logical(from)) storage.mode(from) <- "logical"
+    if(anyNA(from)) stop("cannot coerce NA values to pattern \"ngCMatrix\"")
+    .Call(matrix_to_Csparse, from, "ngCMatrix")
+}
+setAs("matrix", "dgCMatrix", .m2dgC)
+setAs("matrix", "lgCMatrix", .m2lgC)
+setAs("matrix", "ngCMatrix", .m2ngC)
+
+setAs("matrix", "CsparseMatrix", ## => choosing 'l*' or 'dgCMatrix' (no tri-, sym-, diag-):
+      function(from) (if(is.logical(from)) .m2lgC else .m2dgC)(from))
+
+setAs("numeric", "CsparseMatrix",
+      function(from) (if(is.logical(from)) .m2lgC else .m2dgC)(as.matrix.default(from)))
+
+
 setAs("CsparseMatrix", "symmetricMatrix",
       function(from) {
 	  if(isSymmetric(from)) forceCspSymmetric(from)
@@ -86,44 +116,6 @@ setAs("CsparseMatrix", "symmetricMatrix",
 
 ### Some group methods:
 
-setMethod("Math",
-	  signature(x = "CsparseMatrix"),
-	  function(x) {
-	      f0 <- callGeneric(0.)
-	      if(is0(f0)) {
-		  ## sparseness, symm., triang.,... preserved
-                  cl <- class(x)
-                  has.x <- !extends(cl, "nsparseMatrix")
-                  ## has.x  <==> *not* nonzero-pattern == "nMatrix"
-                  if(has.x) {
-                      type <- storage.mode(x@x)
-                      r <- callGeneric(x@x)
-                  } else { ## nsparseMatrix
-                      type <- ""
-		      r <- rep.int(as.double(callGeneric(TRUE)),
-				   switch(.sp.class(cl),
-					  CsparseMatrix = length(x@i),
-					  TsparseMatrix = length(x@i),
-					  RsparseMatrix = length(x@j)))
-                  }
-		  if(type == storage.mode(r)) {
-		      x@x <- r
-		      x
-		  } else { ## e.g. abs( <lgC> ) --> integer Csparse
-		      ## FIXME: when we have 'i*' classes, use them here:
-		      rx <- new(sub("^.", "d", cl))
-		      rx@x <- as.double(r)
-		      ## result is "same"
-		      sNams <- slotNames(cl)
-		      for(nm in sNams[sNams != "x"])
-			  slot(rx, nm) <- slot(x, nm)
-		      rx
-		  }
-	      } else { ## no sparseness
-		  callGeneric(as_dense(x))
-	      }
-	  }) ## {Math}
-
 
 ### Subsetting -- basic things (drop = "missing") are done in ./Matrix.R
 ### ---------- "["  and (currently) also ./sparseMatrix.R
@@ -136,7 +128,10 @@ subCsp_cols <- function(x, j, drop)
     r <- .Call(Csparse_submatrix, x, NULL, jj)
     if(!is.null(n <- dn[[1]])) r@Dimnames[[1]] <- n
     if(!is.null(n <- dn[[2]])) r@Dimnames[[2]] <- n[jj+1L]
-    if(drop && any(r@Dim == 1L)) drop(as(r, "matrix")) else r
+    if(drop && any(r@Dim == 1L)) drop(as(r, "matrix")) else {
+	if(!is.null(n <- names(dn))) names(r@Dimnames) <- n
+	r
+    }
 }
 
 subCsp_rows <- function(x, i, drop)# , cl = getClassDef(class(x))
@@ -147,7 +142,10 @@ subCsp_rows <- function(x, i, drop)# , cl = getClassDef(class(x))
     r <- .Call(Csparse_submatrix, x, ii, NULL)
     if(!is.null(n <- dn[[1]])) r@Dimnames[[1]] <- n[ii+1L]
     if(!is.null(n <- dn[[2]])) r@Dimnames[[2]] <- n
-    if(drop && any(r@Dim == 1L)) drop(as(r, "matrix")) else r
+    if(drop && any(r@Dim == 1L)) drop(as(r, "matrix")) else {
+	if(!is.null(n <- names(dn))) names(r@Dimnames) <- n
+	r
+    }
 }
 
 subCsp_ij <- function(x, i, j, drop)
@@ -164,17 +162,23 @@ subCsp_ij <- function(x, i, j, drop)
     if(!is.null(n <- dn[[1]])) r@Dimnames[[1]] <- n[ii + 1L]
     if(!is.null(n <- dn[[2]])) r@Dimnames[[2]] <- n[jj + 1L]
     if(!i.eq.j) {
-	if(drop && any(r@Dim == 1L)) drop(as(r, "matrix")) else r
+	if(drop && any(r@Dim == 1L)) drop(as(r, "matrix")) else {
+	    if(!is.null(n <- names(dn))) names(r@Dimnames) <- n
+	    r
+	}
     } else { ## i == j
 	if(drop) drop <- any(r@Dim == 1L)
 	if(drop)
 	    drop(as(r, "matrix"))
-	else if(extends((cx <- getClassDef(class(x))),
-                        "symmetricMatrix")) ## TODO? make more efficient:
-	    .gC2sym(r, uplo = x@uplo)## preserve uplo !
-	else if(extends(cx, "triangularMatrix") && !is.unsorted(ii))
-	    as(r, "triangularMatrix")
-	else r
+	else {
+	    if(!is.null(n <- names(dn))) names(r@Dimnames) <- n
+	    if(extends((cx <- getClassDef(class(x))),
+		       "symmetricMatrix")) ## TODO? make more efficient:
+		.gC2sym(r, uplo = x@uplo)## preserve uplo !
+	    else if(extends(cx, "triangularMatrix") && !is.unsorted(ii))
+		as(r, "triangularMatrix")
+	    else r
+	}
     }
 }
 
@@ -216,12 +220,11 @@ replCmat <- function (x, i, j, ..., value)
     dn <- dimnames(x)
     iMi <- missing(i)
     jMi <- missing(j)
-    spV <- is(value, "sparseVector")
     na <- nargs()
     Matrix.msg("replCmat[x,i,j,.., val] : nargs()=", na,"; ",
 	       if(iMi | jMi) sprintf("missing (i,j) = (%d,%d)", iMi,jMi),
 	       .M.level = 2)
-    if(na == 3) { ## "vector (or 2-col) indexing"  M[i] <- v
+    if(na == 3) { ## vector (or 2-col) indexing M[i] <- v : includes M[TRUE] <- v or M[] <- v !
 	x <- as(x, "TsparseMatrix")
 	x[i] <- value # may change class e.g. from dtT* to dgT*
 	clx <- sub(".Matrix$", "CMatrix", (c.x <- class(x)))
@@ -233,7 +236,7 @@ replCmat <- function (x, i, j, ..., value)
 	replCmat4(x,
 		  i1 = if(iMi) 0:(di[1] - 1L) else .ind.prep2(i, 1, di, dn),
 		  i2 = if(jMi) 0:(di[2] - 1L) else .ind.prep2(j, 2, di, dn),
-                  iMi=iMi, jMi=jMi, value=value, spV=spV)
+                  iMi=iMi, jMi=jMi, value=value)
 } ## replCmat
 
 replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector"))
@@ -293,7 +296,7 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
     if(extends(clDx, "dMatrix")) {
 	has.x <- TRUE
 	x <- .Call(dCsparse_subassign,
-		   if(clx %in% c("dgCMatrix", "dtCMatrix"))x
+		   if(clx %in% c("dgCMatrix", "dtCMatrix")) x
 		   else as(x, "dgCMatrix"),
 		   i1, i2,
 		   as(value, "sparseVector"))
@@ -301,7 +304,7 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
     else if(extends(clDx, "lMatrix")) {
 	has.x <- TRUE
 	x <- .Call(lCsparse_subassign,
-		   if(clx %in% c("lgCMatrix", "ltCMatrix"))x
+		   if(clx %in% c("lgCMatrix", "ltCMatrix")) x
 		   else as(x, "lgCMatrix"),
 		   i1, i2,
 		   as(value, "sparseVector"))
@@ -336,9 +339,8 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
 	if(identical(Sys.getenv("USER"),"maechler"))## does it still happen? __ FIXME __
 	    stop("using	 \"old code\" part in  Csparse subassignment")
         ## else
-	warning("using	\"old code\" part in  Csparse subassignment\n",
-		" >>> please report to Matrix-authors@r-project.org",
-		immediate. = TRUE, domain = NA)
+	warning("using\"old code\" part in Csparse subassignment\n >>> please report to Matrix-authors@r-project.org",
+		immediate. = TRUE)
 
 	xj <- .Call(Matrix_expand_pointers, x@p)
 	sel <- (!is.na(match(x@i, i1)) &
@@ -349,8 +351,8 @@ replCmat4 <- function(x, i1, i2, iMi, jMi, value, spV = is(value,"sparseVector")
 	if(has.x && sum(sel) == lenRepl) { ## all entries to be replaced are non-zero:
 	    ## need indices instead of just 'sel', for, e.g.,  A[2:1, 2:1] <- v
 	    non0 <- cbind(match(x@i[sel], i1),
-			  match(xj [sel], i2)) - 1L
-	    iN0 <- 1L + .Call(m_encodeInd, non0, di = dind, checkBounds = FALSE)
+			  match(xj [sel], i2), deparse.level=0L)
+	    iN0 <- 1L + .Call(m_encodeInd, non0, di = dind, orig1=TRUE, checkBounds=FALSE)
 
 	    has0 <-
 		if(spV) length(value@i) < lenV else any(value[!is.na(value)] == 0)
@@ -437,7 +439,7 @@ setMethod("t", signature(x = "CsparseMatrix"),
 
 
 ## NB: have extra tril(), triu() methods for symmetric ["dsC" and "lsC"] and
-## NB: for all triangular ones, where the latter may 'callNextMethod()' these:
+##     for all triangular ones, where the latter may 'callNextMethod()' these:
 setMethod("tril", "CsparseMatrix",
 	  function(x, k = 0, ...) {
 	      k <- as.integer(k[1])
@@ -445,8 +447,7 @@ setMethod("tril", "CsparseMatrix",
 	      stopifnot(-dd[1] <= k, k <= dd[1]) # had k <= 0
 	      r <- .Call(Csparse_band, x, -dd[1], k)
 	      ## return "lower triangular" if k <= 0
-	      if(sqr && k <= 0)
-		  as(r, paste0(.M.kind(x), "tCMatrix")) else r
+	      if(sqr && k <= 0) .gC2tC(r, uplo = "L") else r
 	  })
 
 setMethod("triu", "CsparseMatrix",
@@ -456,8 +457,7 @@ setMethod("triu", "CsparseMatrix",
 	      stopifnot(-dd[1] <= k, k <= dd[1]) # had k >= 0
 	      r <- .Call(Csparse_band, x, k, dd[2])
 	      ## return "upper triangular" if k >= 0
-	      if(sqr && k >= 0)
-		  as(r, paste0(.M.kind(x), "tCMatrix")) else r
+	      if(sqr && k >= 0) .gC2tC(r, uplo = "U") else r
 	  })
 
 setMethod("band", "CsparseMatrix",
