@@ -260,7 +260,7 @@ combine_rleD <- function(rleList, m = length(rleList))
 
     ## llen1 <- unlist(lapply(lens, length)) + 1L
     ## n <- sum(llen1)
-    n <- m + sum(unlist(lapply(lens, length)))
+    n <- m + sum(lengths(lens, use.names=FALSE))
     ## comb(): intersperse x2[[j]] between lis[[j] & lis[[j+1]] :
     comb <- function(lis, x2)
         unlist(mapply(c, lis, x2, SIMPLIFY=FALSE, USE.NAMES=FALSE))
@@ -325,10 +325,8 @@ setAs("abIndex", "integer", function(from) as.integer(abI2num(from)))
 ## for S3 lovers and back-compatibility:
 setMethod(as.integer, "abIndex", function(x) as.integer(abI2num(x)))
 setMethod(as.numeric, "abIndex", function(x) abI2num(x))
-setMethod(as.vector,  c(x = "abIndex", mode = "ANY"), function(x) abI2num(x))
-setMethod(as.vector,  c(x = "abIndex", mode = "character"),
-	  ## this is beautiful -- because of as() !
-	  function(x, mode) as(abI2num(x), mode))
+setMethod("as.vector", "abIndex",
+	  function(x, mode) as.vector(abI2num(x), mode))
 
 ## Need   max(<i>), min(<i>),   all(<i> == <j>)   any(<i> == <j>)
 
@@ -340,13 +338,14 @@ setMethod("[", signature(x = "abIndex", i = "index"),
 	  function (x, i, j, ..., drop)
       {
           switch(x@kind,
-                 "rleDiff" = {
-                     ## intIv() in ./sparseVector.R -- not memory-efficient (??)
-                     n <- length(x)
-                     ii <- intIv(i, n) ## ii : 1-based integer indices
-                     d <- x@rleD
-                     ## Now work with the equivalent of
-                     ##   cumsum(c(d@first, rep.int(d@rle$values, d@rle$lengths)))
+		 "rleDiff" = {
+		     ## FIXME
+		     ## intIv() in ./sparseVector.R -- not memory-efficient (??)
+		     ## n <- length(x)
+		     ## ii <- intIv(i, n) ## ii : 1-based integer indices
+		     ## d <- x@rleD
+		     ## Now work with the equivalent of
+		     ##   cumsum(c(d@first, rep.int(d@rle$values, d@rle$lengths)))
 
                      stop("<abIndex>[i]  is not yet implemented")
                  },
@@ -423,7 +422,7 @@ setMethod("Summary", signature(x = "abIndex", na.rm = "ANY"),
                      switch(.Generic,
                             "all" = {
                                 ## these often, but *not* always come in pairs
-                                en <- ends.rleD(d)
+				## en <- ends.rleD(d)
                                 ## so maybe it does not really help!
 
                                 stop("all(<abIndex>) is not yet implemented")
@@ -519,7 +518,7 @@ setMethod("Arith", signature(e1 = "abIndex", e2 = "numLike"),
 		     },
 		     "/" = {
                          if(is0(e2) ## division by 0
-                            && length(unique(sign(ee <- ends.rleD(e1@rleD)))) > 1) {
+                            && length(unique(sign(ends.rleD(e1@rleD)))) > 1) {
                              ## at least one subsequence contains 0, i.e., changes sign:
 			     warning("x / 0 for an <abIndex> x with sign-change\n no longer representable as 'rleDiff'")
                              return(vec2abI(abI2num(e1) / 0))
@@ -692,6 +691,7 @@ setAs("abIndex", "seqMat", function(from)
       ## we need to care for the "length 1" stretches:
       if(any(nonPair <- le[2* seq_len(m2 <- m %/% 2)] != 1)) {
 
+          m2 + n + va + nonPair # <- "dummy" using "unused"
           ## an "easy" (but not so efficient when 'm' is "large")
           ## way would be to "make these" into pairs, then work for that case...
       }
