@@ -1,12 +1,13 @@
-stopifnot(require(Matrix)) # at least its classes
+stopifnot(requireNamespace("methods", quietly = TRUE),
+	  requireNamespace("Matrix" , quietly = TRUE))
 
 CAex <-
     local({
 	load(system.file(file.path("external", "CAex_slots.rda"), package = "Matrix"))
 	## -> 'L'
-	r <- new("dgCMatrix")
+	r <- methods::new("dgCMatrix")
 	for (n in c("Dim", "i","p","x"))
-	    slot(r, n) <- L[[n]]
+	    methods::slot(r, n) <- L[[n]]
 	r
     })
 
